// Copyright (C) 2022 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'package:bloc/bloc.dart';
import 'package:equatable/equatable.dart';
import 'package:wyatt_authentication_bloc/src/authentication/cubit/authentication_cubit.dart';
import 'package:wyatt_authentication_bloc/src/models/exceptions/exceptions_interface.dart';
import 'package:wyatt_authentication_bloc/src/repositories/authentication_repository_interface.dart';
import 'package:wyatt_form_bloc/wyatt_form_bloc.dart';

part 'sign_in_state.dart';

class SignInCubit extends Cubit<SignInState> {
  final AuthenticationRepositoryInterface _authenticationRepository;
  final AuthenticationCubit _authenticationCubit;

  SignInCubit({
    required AuthenticationRepositoryInterface authenticationRepository,
    required AuthenticationCubit authenticationCubit,
  })  : _authenticationRepository = authenticationRepository,
        _authenticationCubit = authenticationCubit,
        super(const SignInState());

  void emailChanged(String value) {
    final Email email = Email.dirty(value);
    emit(
      state.copyWith(
        email: email,
        status: FormData.validate([email, state.password]),
      ),
    );
  }

  void passwordChanged(String value) {
    final Password password = Password.dirty(value);
    emit(
      state.copyWith(
        password: password,
        status: FormData.validate([state.email, password]),
      ),
    );
  }

  Future<void> signInAnonymously() async {
    if (state.status.isSubmissionInProgress) {
      return;
    }

    emit(state.copyWith(status: FormStatus.submissionInProgress));
    try {
      await _authenticationRepository.signInAnonymously();
      _authenticationCubit.start();
      emit(state.copyWith(status: FormStatus.submissionSuccess));
    } on SignInAnonymouslyFailureInterface catch (e) {
      emit(
        state.copyWith(
          errorMessage: e.message,
          status: FormStatus.submissionFailure,
        ),
      );
    } catch (_) {
      emit(state.copyWith(status: FormStatus.submissionFailure));
    }
  }

  Future<void> signInWithGoogle() async {
    if (state.status.isSubmissionInProgress) {
      return;
    }

    emit(state.copyWith(status: FormStatus.submissionInProgress));
    try {
      await _authenticationRepository.signInWithGoogle();
      _authenticationCubit.start();
      emit(state.copyWith(status: FormStatus.submissionSuccess));
    } on SignInWithGoogleFailureInterface catch (e) {
      emit(
        state.copyWith(
          errorMessage: e.message,
          status: FormStatus.submissionFailure,
        ),
      );
    } catch (_) {
      emit(state.copyWith(status: FormStatus.submissionFailure));
    }
  }

  Future<void> signInWithEmailAndPassword() async {
    if (!state.status.isValidated) return;
    emit(state.copyWith(status: FormStatus.submissionInProgress));
    try {
      await _authenticationRepository.signInWithEmailAndPassword(
        email: state.email.value,
        password: state.password.value,
      );
      _authenticationCubit.start();
      emit(state.copyWith(status: FormStatus.submissionSuccess));
    } on SignInWithEmailAndPasswordFailureInterface catch (e) {
      emit(
        state.copyWith(
          errorMessage: e.message,
          status: FormStatus.submissionFailure,
        ),
      );
    } catch (_) {
      emit(state.copyWith(status: FormStatus.submissionFailure));
    }
  }
}
